@@ -66,22 +66,14 @@ class CachedSequenceMatcher(object):
             if os.name == "nt":
                 CachedSequenceMatcher.process_pool = ThreadPool(None)
             else:
-                # maxtasksperchild is new in Python 2.7; this is for 2.6 compat
-                try:
-                    CachedSequenceMatcher.process_pool = Pool(
-                        None, matchers.init_worker, maxtasksperchild=1)
-                except TypeError:
-                    CachedSequenceMatcher.process_pool = Pool(
-                        None, matchers.init_worker)
+                CachedSequenceMatcher.process_pool = Pool(
+                    None, matchers.init_worker, maxtasksperchild=1)
         self.cache = {}
 
     def match(self, text1, textn, cb):
         try:
             self.cache[(text1, textn)][1] = time.time()
             opcodes = self.cache[(text1, textn)][0]
-            # FIXME: This idle should be totally unnecessary, and yet nothing
-            # gets highlighted without it, even though everything in the
-            # callback appears to run identically.
             GLib.idle_add(lambda: cb(opcodes))
         except KeyError:
             def inline_cb(opcodes):
@@ -182,7 +174,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         """Start up an filediff with num_panes empty contents.
         """
         melddoc.MeldDoc.__init__(self)
-        gnomeglade.Component.__init__(self, "filediff.ui", "filediff")
+        gnomeglade.Component.__init__(
+            self, "filediff.ui", "filediff", ["FilediffActions"])
         bind_settings(self)
 
         widget_lists = [
@@ -247,88 +240,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf.connect("notify::has-selection",
                         self.update_text_actions_sensitivity)
 
-        actions = (
-            ("MakePatch", None, _("Format as Patch..."), None,
-                _("Create a patch using differences between files"),
-                self.make_patch),
-            ("SaveAll", None, _("Save A_ll"), "<Ctrl><Shift>L",
-                _("Save all files in the current comparison"),
-                self.on_save_all_activate),
-            ("Revert", Gtk.STOCK_REVERT_TO_SAVED, None, None,
-                _("Revert files to their saved versions"),
-                self.on_revert_activate),
-            ("SplitAdd", None, _("Add Synchronization Point"), None,
-                _("Add a manual point for synchronization of changes between "
-                  "files"),
-                self.add_sync_point),
-            ("SplitClear", None, _("Clear Synchronization Points"), None,
-                _("Clear manual change sychronization points"),
-                self.clear_sync_points),
-            ("PrevConflict", None, _("Previous Conflict"), "<Ctrl>I",
-                _("Go to the previous conflict"),
-                lambda x: self.on_next_conflict(Gdk.ScrollDirection.UP)),
-            ("NextConflict", None, _("Next Conflict"), "<Ctrl>K",
-                _("Go to the next conflict"),
-                lambda x: self.on_next_conflict(Gdk.ScrollDirection.DOWN)),
-            ("PushLeft", Gtk.STOCK_GO_BACK, _("Push to Left"), "<Alt>Left",
-                _("Push current change to the left"),
-                lambda x: self.push_change(-1)),
-            ("PushRight", Gtk.STOCK_GO_FORWARD,
-                _("Push to Right"), "<Alt>Right",
-                _("Push current change to the right"),
-                lambda x: self.push_change(1)),
-            # FIXME: using LAST and FIRST is terrible and unreliable icon abuse
-            ("PullLeft", Gtk.STOCK_GOTO_LAST,
-                _("Pull from Left"), "<Alt><Shift>Right",
-                _("Pull change from the left"),
-                lambda x: self.pull_change(-1)),
-            ("PullRight", Gtk.STOCK_GOTO_FIRST,
-                _("Pull from Right"), "<Alt><Shift>Left",
-                _("Pull change from the right"),
-                lambda x: self.pull_change(1)),
-            ("CopyLeftUp", None, _("Copy Above Left"), "<Alt>bracketleft",
-                _("Copy change above the left chunk"),
-                lambda x: self.copy_change(-1, -1)),
-            ("CopyLeftDown", None, _("Copy Below Left"), "<Alt>semicolon",
-                _("Copy change below the left chunk"),
-                lambda x: self.copy_change(-1, 1)),
-            ("CopyRightUp", None, _("Copy Above Right"), "<Alt>bracketright",
-                _("Copy change above the right chunk"),
-                lambda x: self.copy_change(1, -1)),
-            ("CopyRightDown", None, _("Copy Below Right"), "<Alt>quoteright",
-                _("Copy change below the right chunk"),
-                lambda x: self.copy_change(1, 1)),
-            ("Delete", Gtk.STOCK_DELETE, _("Delete"), "<Alt>Delete",
-                _("Delete change"),
-                self.delete_change),
-            ("MergeFromLeft", None, _("Merge All from Left"), None,
-                _("Merge all non-conflicting changes from the left"),
-                lambda x: self.pull_all_non_conflicting_changes(-1)),
-            ("MergeFromRight", None, _("Merge All from Right"), None,
-                _("Merge all non-conflicting changes from the right"),
-                lambda x: self.pull_all_non_conflicting_changes(1)),
-            ("MergeAll", None, _("Merge All"), None,
-                _("Merge all non-conflicting changes from left and right "
-                  "panes"),
-                lambda x: self.merge_all_non_conflicting_changes()),
-            ("CycleDocuments", None,
-                _("Cycle Through Documents"), "<control>Escape",
-                _("Move keyboard focus to the next document in this "
-                  "comparison"),
-                self.action_cycle_documents),
-        )
-
-        toggle_actions = (
-            ("LockScrolling", None, _("Lock Scrolling"), None,
-             _("Lock scrolling of all panes"),
-             self.on_action_lock_scrolling_toggled, True),
-        )
-
         self.ui_file = gnomeglade.ui_file("filediff-ui.xml")
-        self.actiongroup = Gtk.ActionGroup(name="FilediffPopupActions")
+        self.actiongroup = self.FilediffActions
         self.actiongroup.set_translation_domain("meld")
-        self.actiongroup.add_actions(actions)
-        self.actiongroup.add_toggle_actions(toggle_actions)
 
         self.findbar = findbar.FindBar(self.grid)
         self.grid.attach(self.findbar.widget, 1, 2, 5, 1)
@@ -403,6 +317,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             if event.keyval == Gdk.KEY_Escape:
                 self.findbar.hide()
         elif event.type == Gdk.EventType.KEY_RELEASE:
+            if event.keyval == Gdk.KEY_Return and self.keymask & MASK_SHIFT:
+                self.findbar.start_find_previous(self.focus_pane)
             self.keymask &= ~mod_key
 
     def on_style_updated(self, widget):
@@ -419,6 +335,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             tag = buf.get_tag_table().lookup("inline")
             tag.props.background_rgba = lookup("inline-bg", "LightSteelBlue2")
 
+        override_bg = style.lookup_color("override-background-color")
+        self.override_bg = override_bg[1] if override_bg[0] else None
+
         self.fill_colors = {"insert"  : lookup("insert-bg", "DarkSeaGreen1"),
                             "delete"  : lookup("insert-bg", "DarkSeaGreen1"),
                             "conflict": lookup("conflict-bg", "Pink"),
@@ -552,13 +471,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 chunk = self.linediffer.get_chunk(chunk_id, pane)
                 insert_chunk = chunk[1] == chunk[2]
                 delete_chunk = chunk[3] == chunk[4]
-                push_left = editable_left and not insert_chunk
-                push_right = editable_right and not insert_chunk
+                push_left = editable_left
+                push_right = editable_right
                 pull_left = pane == 2 and editable and not delete_chunk
                 pull_right = pane == 0 and editable and not delete_chunk
                 delete = editable and not insert_chunk
-                copy_left = push_left and not delete_chunk
-                copy_right = push_right and not delete_chunk
+                copy_left = editable_left and not (insert_chunk or delete_chunk)
+                copy_right = editable_right and not (insert_chunk or delete_chunk)
             elif pane == 1:
                 chunk0 = self.linediffer.get_chunk(chunk_id, 1, 0)
                 chunk2 = None
@@ -568,13 +487,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 left_exists = chunk0 is not None and chunk0[3] != chunk0[4]
                 right_mid_exists = chunk2 is not None and chunk2[1] != chunk2[2]
                 right_exists = chunk2 is not None and chunk2[3] != chunk2[4]
-                push_left = editable_left and left_mid_exists
-                push_right = editable_right and right_mid_exists
+                push_left = editable_left
+                push_right = editable_right
                 pull_left = editable and left_exists
                 pull_right = editable and right_exists
                 delete = editable and (left_mid_exists or right_mid_exists)
-                copy_left = push_left and left_exists
-                copy_right = push_right and right_exists
+                copy_left = editable_left and left_mid_exists and left_exists
+                copy_right = editable_right and right_mid_exists and right_exists
         self.actiongroup.get_action("PushLeft").set_sensitive(push_left)
         self.actiongroup.get_action("PushRight").set_sensitive(push_right)
         self.actiongroup.get_action("PullLeft").set_sensitive(pull_left)
@@ -584,6 +503,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.get_action("CopyLeftDown").set_sensitive(copy_left)
         self.actiongroup.get_action("CopyRightUp").set_sensitive(copy_right)
         self.actiongroup.get_action("CopyRightDown").set_sensitive(copy_right)
+
+        prev_pane = pane > 0
+        next_pane = pane < self.num_panes - 1
+        self.actiongroup.get_action("PrevPane").set_sensitive(prev_pane)
+        self.actiongroup.get_action("NextPane").set_sensitive(next_pane)
         # FIXME: don't queue_draw() on everything... just on what changed
         self.queue_draw()
 
@@ -591,54 +515,98 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.get_action("PrevConflict").set_sensitive(have_prev)
         self.actiongroup.get_action("NextConflict").set_sensitive(have_next)
 
-    def on_next_conflict(self, direction):
-        if direction == Gdk.ScrollDirection.DOWN:
-            target = self.cursor.next_conflict
-        else: # direction == Gdk.ScrollDirection.UP
-            target = self.cursor.prev_conflict
-
+    def go_to_chunk(self, target, pane=None, centered=False):
         if target is None:
             return
 
-        buf = self.textbuffer[self.cursor.pane]
-        chunk = self.linediffer.get_chunk(target, self.cursor.pane)
-        buf.place_cursor(buf.get_iter_at_line(chunk[1]))
-        self.textview[self.cursor.pane].scroll_to_mark(
-            buf.get_insert(), 0.1, True, 0.5, 0.5)
+        if pane is None:
+            pane = self._get_focused_pane()
+            if pane == -1:
+                pane = 1 if len(self.textview) > 1 else 0
 
-    def push_change(self, direction):
+        chunk = self.linediffer.get_chunk(target, pane)
+        if not chunk:
+            return
+
+        # Warp the cursor to the first line of the chunk
+        buf = self.textbuffer[pane]
+        if self.cursor.line != chunk[1]:
+            buf.place_cursor(buf.get_iter_at_line(chunk[1]))
+
+        tolerance = 0.0 if centered else 0.2
+        self.textview[pane].scroll_to_mark(
+            buf.get_insert(), tolerance, True, 0.5, 0.5)
+
+    def next_diff(self, direction, centered=False):
+        target = (self.cursor.next if direction == Gdk.ScrollDirection.DOWN
+                  else self.cursor.prev)
+        self.go_to_chunk(target, centered=centered)
+
+    def action_previous_conflict(self, *args):
+        self.go_to_chunk(self.cursor.prev_conflict, self.cursor.pane)
+
+    def action_next_conflict(self, *args):
+        self.go_to_chunk(self.cursor.next_conflict, self.cursor.pane)
+
+    def action_previous_diff(self, *args):
+        self.go_to_chunk(self.cursor.prev)
+
+    def action_next_diff(self, *args):
+        self.go_to_chunk(self.cursor.next)
+
+    def get_action_chunk(self, src, dst):
+        valid_panes = list(range(0, self.num_panes))
+        if (src not in valid_panes or dst not in valid_panes or
+                self.cursor.chunk is None):
+            raise ValueError("Action was taken on invalid panes")
+
+        chunk = self.linediffer.get_chunk(self.cursor.chunk, src, dst)
+        if chunk is None:
+            raise ValueError("Action was taken on a missing chunk")
+        return chunk
+
+    def get_action_panes(self, direction, reverse=False):
         src = self._get_focused_pane()
         dst = src + direction
-        chunk = self.linediffer.get_chunk(self.cursor.chunk, src, dst)
-        assert(src != -1 and self.cursor.chunk is not None)
-        assert(dst in (0, 1, 2))
-        assert(chunk is not None)
-        self.replace_chunk(src, dst, chunk)
+        return (dst, src) if reverse else (src, dst)
 
-    def pull_change(self, direction):
-        dst = self._get_focused_pane()
-        src = dst + direction
-        chunk = self.linediffer.get_chunk(self.cursor.chunk, src, dst)
-        assert(dst != -1 and self.cursor.chunk is not None)
-        assert(src in (0, 1, 2))
-        assert(chunk is not None)
-        self.replace_chunk(src, dst, chunk)
+    def action_push_change_left(self, *args):
+        src, dst = self.get_action_panes(-1)
+        self.replace_chunk(src, dst, self.get_action_chunk(src, dst))
 
-    def copy_change(self, direction, copy_direction):
-        src = self._get_focused_pane()
-        dst = src + direction
-        chunk = self.linediffer.get_chunk(self.cursor.chunk, src, dst)
-        assert(src != -1 and self.cursor.chunk is not None)
-        assert(dst in (0, 1, 2))
-        assert(chunk is not None)
-        copy_up = True if copy_direction < 0 else False
-        self.copy_chunk(src, dst, chunk, copy_up)
+    def action_push_change_right(self, *args):
+        src, dst = self.get_action_panes(+1)
+        self.replace_chunk(src, dst, self.get_action_chunk(src, dst))
 
-    def pull_all_non_conflicting_changes(self, direction):
-        assert direction in (-1, 1)
-        dst = self._get_focused_pane()
-        src = dst + direction
-        assert src in range(self.num_panes)
+    def action_pull_change_left(self, *args):
+        src, dst = self.get_action_panes(-1, reverse=True)
+        self.replace_chunk(src, dst, self.get_action_chunk(src, dst))
+
+    def action_pull_change_right(self, *args):
+        src, dst = self.get_action_panes(+1, reverse=True)
+        self.replace_chunk(src, dst, self.get_action_chunk(src, dst))
+
+    def action_copy_change_left_up(self, *args):
+        src, dst = self.get_action_panes(-1)
+        self.copy_chunk(
+            src, dst, self.get_action_chunk(src, dst), copy_up=True)
+
+    def action_copy_change_right_up(self, *args):
+        src, dst = self.get_action_panes(+1)
+        self.copy_chunk(
+            src, dst, self.get_action_chunk(src, dst), copy_up=True)
+
+    def action_copy_change_left_down(self, *args):
+        src, dst = self.get_action_panes(-1)
+        self.copy_chunk(
+            src, dst, self.get_action_chunk(src, dst), copy_up=False)
+
+    def action_copy_change_right_down(self, *args):
+        src, dst = self.get_action_panes(+1)
+        self.copy_chunk(
+            src, dst, self.get_action_chunk(src, dst), copy_up=False)
+
+    def pull_all_non_conflicting_changes(self, src, dst):
         merger = merge.Merger()
         merger.differ = self.linediffer
         merger.texts = self.buffer_texts
@@ -648,11 +616,20 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.on_textbuffer_begin_user_action()
         self.textbuffer[dst].set_text(mergedfile)
         self.on_textbuffer_end_user_action()
+
         def resync():
             self._sync_vscroll_lock = False
             self._sync_vscroll(self.scrolledwindow[src].get_vadjustment(), src)
         self.scheduler.add_task(resync)
 
+    def action_pull_all_changes_left(self, *args):
+        src, dst = self.get_action_panes(-1, reverse=True)
+        self.pull_all_non_conflicting_changes(src, dst)
+
+    def action_pull_all_changes_right(self, *args):
+        src, dst = self.get_action_panes(+1, reverse=True)
+        self.pull_all_non_conflicting_changes(src, dst)
+
     def merge_all_non_conflicting_changes(self):
         dst = 1
         merger = merge.Merger()
@@ -762,11 +739,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
         return new_line
 
-    def action_cycle_documents(self, widget):
-        pane = self._get_focused_pane()
-        new_pane = (pane + 1) % self.num_panes
+    def move_cursor_pane(self, pane, new_pane):
         chunk, line = self.cursor.chunk, self.cursor.line
-
         new_line = self._corresponding_chunk_line(chunk, line, pane, new_pane)
 
         new_buf = self.textbuffer[new_pane]
@@ -775,6 +749,16 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.textview[new_pane].scroll_to_mark(
             new_buf.get_insert(), 0.1, True, 0.5, 0.5)
 
+    def action_prev_pane(self, *args):
+        pane = self._get_focused_pane()
+        new_pane = (pane - 1) % self.num_panes
+        self.move_cursor_pane(pane, new_pane)
+
+    def action_next_pane(self, *args):
+        pane = self._get_focused_pane()
+        new_pane = (pane + 1) % self.num_panes
+        self.move_cursor_pane(pane, new_pane)
+
     def _set_external_action_sensitivity(self):
         have_file = self.focus_pane is not None
         try:
@@ -871,7 +855,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             # but this is unbound on currently required PyGTK.
             buttons = []
             for i in range(self.num_panes):
-                button = Gtk.CheckButton(self.textbuffer[i].data.label)
+                button = Gtk.CheckButton(label=self.textbuffer[i].data.label)
                 button.set_use_underline(False)
                 button.set_sensitive(modified[i])
                 button.set_active(modified[i])
@@ -916,14 +900,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if response == Gtk.ResponseType.OK:
             for h in self.settings_handlers:
                 meldsettings.disconnect(h)
-        # TODO: Base the return code on something meaningful for VC tools
-        self.emit('close', 0)
+            # TODO: Base the return code on something meaningful for VC tools
+            self.emit('close', 0)
         return response
 
-        #
-        # text buffer undo/redo
-        #
-
     def on_undo_activate(self):
         if self.undosequence.can_undo():
             self.undosequence.undo()
@@ -954,10 +934,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_undo_checkpointed(self, undosequence, buf, checkpointed):
         self.set_buffer_modified(buf, not checkpointed)
 
-        #
-        #
-        #
-
     def open_external(self):
         pane = self._get_focused_pane()
         if pane >= 0:
@@ -1037,11 +1013,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.textview_overwrite_handlers = [ t.connect("toggle-overwrite", self.on_textview_toggle_overwrite) for t in self.textview ]
         self.on_cursor_position_changed(view.get_buffer(), None, True)
 
-
-        #
-        # text buffer loading/saving
-        #
-
     def set_labels(self, labels):
         labels = labels[:len(self.textbuffer)]
         for label, buf in zip(labels, self.textbuffer):
@@ -1225,7 +1196,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
             if self.cursor.next is not None:
                 self.scheduler.add_task(
-                    lambda: self.next_diff(Gdk.ScrollDirection.DOWN, True), True)
+                    lambda: self.go_to_chunk(self.cursor.next, centered=True),
+                    True)
             else:
                 buf = self.textbuffer[1 if self.num_panes > 1 else 0]
                 self.on_cursor_position_changed(buf, None, True)
@@ -1391,6 +1363,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     if texts != (text1, textn):
                         return
 
+                    if clear:
+                        bufs[0].remove_tag(tags[0], starts[0], ends[0])
+                        bufs[1].remove_tag(tags[1], starts[1], ends[1])
+
                     offsets = [ends[0].get_offset() - starts[0].get_offset(),
                                ends[1].get_offset() - starts[1].get_offset()]
 
@@ -1418,10 +1394,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             end.set_offset(offset + o[2 + 2 * i])
                             bufs[i].apply_tag(tags[i], start, end)
 
-                if clear:
-                    bufs[0].remove_tag(tags[0], starts[0], ends[0])
-                    bufs[1].remove_tag(tags[1], starts[1], ends[1])
-
                 starts = [bufs[0].create_mark(None, starts[0], True),
                           bufs[1].create_mark(None, starts[1], True)]
                 ends = [bufs[0].create_mark(None, ends[0], True),
@@ -1529,6 +1501,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         width, height = view_allocation.width, view_allocation.height
         context.set_line_width(1.0)
 
+        if self.override_bg:
+            context.set_source_rgba(*self.override_bg)
+            context.rectangle(0, 0, width, height)
+            context.fill()
+
         for change in self.linediffer.single_changes(pane, bounds):
             ypos0 = textview.get_y_for_line_num(change[1]) - visible.y
             ypos1 = textview.get_y_for_line_num(change[2]) - visible.y
@@ -1538,8 +1515,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 context.set_source_rgba(*self.fill_colors[change[0]])
                 context.fill_preserve()
                 if self.linediffer.locate_chunk(pane, change[1])[0] == self.cursor.chunk:
-                    h = self.fill_colors['current-chunk-highlight']
-                    context.set_source_rgba(h.red, h.green, h.blue, 0.5)
+                    highlight = self.fill_colors['current-chunk-highlight']
+                    context.set_source_rgba(*highlight)
                     context.fill_preserve()
 
             context.set_source_rgba(*self.line_colors[change[0]])
@@ -1762,11 +1739,15 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.save_file(idx)
 
     def on_fileentry_file_set(self, entry):
+        entries = self.fileentry[:self.num_panes]
         if self.check_save_modified() != Gtk.ResponseType.CANCEL:
-            entries = self.fileentry[:self.num_panes]
             files = [e.get_file() for e in entries]
             paths = [f.get_path() for f in files]
             self.set_files(paths)
+        else:
+            idx = entries.index(entry)
+            existing_path = self.textbuffer[idx].data.filename
+            entry.set_filename(existing_path)
         return True
 
     def _get_focused_pane(self):
@@ -1815,9 +1796,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         buf = self.textbuffer[index]
         self.set_buffer_editable(buf, not button.get_active())
 
-        #
-        # scrollbars
-        #
     def _sync_hscroll(self, adjustment):
         if self._sync_hscroll_lock or self._scroll_lock:
             return
@@ -1900,7 +1878,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 widget.hide()
 
             self.actiongroup.get_action("MakePatch").set_sensitive(n > 1)
-            self.actiongroup.get_action("CycleDocuments").set_sensitive(n > 1)
 
             def coords_iter(i):
                 buf_index = 2 if i == 1 and self.num_panes == 3 else i
@@ -1933,31 +1910,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.queue_draw()
             self.recompute_label()
 
-    def next_diff(self, direction, centered=False):
-        target = (self.cursor.next if direction == Gdk.ScrollDirection.DOWN
-                  else self.cursor.prev)
-        if target is None:
-            return
-
-        pane = self._get_focused_pane()
-        if pane == -1:
-            if len(self.textview) > 1:
-                pane = 1
-            else:
-                pane = 0
-
-        chunk = self.linediffer.get_chunk(target, pane)
-        if not chunk:
-            return
-
-        # Warp the cursor to the first line of next chunk
-        buf = self.textbuffer[pane]
-        if self.cursor.line != chunk[1]:
-            buf.place_cursor(buf.get_iter_at_line(chunk[1]))
-        tolerance = 0.0 if centered else 0.2
-        self.textview[pane].scroll_to_mark(
-            buf.get_insert(), tolerance, True, 0.5, 0.5)
-
     def copy_chunk(self, src, dst, chunk, copy_up):
         b0, b1 = self.textbuffer[src], self.textbuffer[dst]
         start = b0.get_iter_at_line_or_eof(chunk[1])
@@ -2001,10 +1953,16 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         new_end = b1.insert_at_line(chunk[3], t0)
         self.on_textbuffer_end_user_action()
         mark1 = b1.create_mark(None, new_end, True)
-        # FIXME: If the inserted chunk ends up being an insert chunk, then
-        # this animation is not visible; this happens often in three-way diffs
-        rgba0 = self.fill_colors['insert'].copy()
-        rgba1 = self.fill_colors['insert'].copy()
+        if chunk[1] == chunk[2]:
+            # TODO: Need a more specific colour here; conflict is wrong
+            rgba0 = self.fill_colors['conflict'].copy()
+            rgba1 = self.fill_colors['conflict'].copy()
+        else:
+            # FIXME: If the inserted chunk ends up being an insert chunk, then
+            # this animation is not visible; this happens often in three-way
+            # diffs
+            rgba0 = self.fill_colors['insert'].copy()
+            rgba1 = self.fill_colors['insert'].copy()
         rgba0.alpha = 1.0
         rgba1.alpha = 0.0
         anim = TextviewLineAnimation(mark0, mark1, rgba0, rgba1, 500000)
